
/*
 * This applet is a simple example of how contact a cgi-script using the
 * POST method. It sends the data in sdata and puts any received data into
 * rdata; rdata is then displayed in the applet.
 *
 * The variable script is the cgi-script to be contacted. Note: since we're
 * doing raw HTTP here the variable script must contain the full (absolute)
 * path to the cgi-script.
 *
 * Note also: The cgi-script must reside on the same machine as where the
 * applet was loaded from (this is a security constraint in Netscape
 * browsers, and will probably be around in other browsers too).
 */

import java.applet.Applet;
import java.awt.Graphics;
import java.net.Socket;
import java.io.*;
import java.util.StringTokenizer;


public class PostExample extends Applet
{
    private String script = "/cgi-bin/post-query";
    private final String ctype  = "application/x-www-form-urlencoded";
//  private final String ctype  = "application/octet-stream";
    private       String sdata  = "Filename=Terrain&Tolerance=0.0";
    private       String rdata  = "";
    private       String rhdrs  = "";
    private       String home;
    private       int    port;


    public void init()
    {
	home = getCodeBase().getHost();
	port = getCodeBase().getPort();
	if (port == -1)  port = 80;
    }


    public void start()
    {
	Socket           sock;
	OutputStream     outp;
	InputStream      inp;
	DataOutputStream dataout;
	DataInputStream  datain;


	rdata = "";
	rhdrs = "";


	//create a client socket 
	try
	{
	    sock = new Socket(home, port);
	}
	catch (Exception e)
	{
	    rdata = e+" (socket: Host: "+home+"\tPort: "+port+")";
	    return;
	}


	// Obtain output stream to communicate with the server
	try
	{
	    outp = sock.getOutputStream();
	    inp  = sock.getInputStream();
	}
	catch (Exception e)
	{
	    rdata = e+" (get IOStream)";
	    try
	    {
		sock.close();
	    }
	    catch (IOException ee) {;}
	    return;
	}

	try
	{
	    dataout = new DataOutputStream(outp);
	    datain  = new DataInputStream(inp);
	}
	catch (Exception e)
	{
	    rdata = e+" (get DataIOStream)";
	    try
	    {
		sock.close();
	    }
	    catch (IOException ee) {;}
	    return;
	}


	// Send http request to server and get return data
	try
	{
	    // HTTP header
	    dataout.writeBytes("POST " + script + " HTTP/1.0\r\n");
	    dataout.writeBytes("Content-type: " + ctype + "\r\n");
            dataout.writeBytes("Content-length: " + sdata.length() + "\r\n");
	    dataout.writeBytes("\r\n");		// end of header
	    // POST data
	    dataout.writeBytes(sdata);

	    // read response headers
	    String  line;

	    while ((line = datain.readLine()) != null  &&  !line.equals(""))
	    {
		rhdrs += line + "\n";
	    }

	    // read response data
	    while ((line = datain.readLine()) != null)
	    {
		rdata += "\n" + line;
	    }
	}
	catch (Exception e)
	{
	    rdata = e+" (read/write data)";
	    try
	    {
		sock.close();
	    }
	    catch (IOException ee) {;}
	    return;
	}


	// close up shop
	try
	{
	    dataout.close();
	    datain.close();
	}
	catch (IOException e) {;}
	try
	{
	    sock.close();
	}
	catch (IOException e) {;}
    }


    public void stop() {}


    public void paint(Graphics g)
    {
	StringTokenizer st;
	int line    = 1,
	    line_sp = getFontMetrics(getFont()).getHeight()+1;

	g.drawString("---Headers:", 5, line*line_sp);  line++;
	st = new StringTokenizer(rhdrs, "\n");
	while (st.hasMoreTokens())
	{
	    g.drawString(st.nextToken(), 5, line*line_sp);
	    line++;
	}

	line++;
	g.drawString("---Data:", 5, line*line_sp);  line++;
	st = new StringTokenizer(rdata, "\n");
	while (st.hasMoreTokens())
	{
	    g.drawString(st.nextToken(), 5, line*line_sp);
	    line++;
	}
    }
}

